#ifndef MATH_H_
#define MATH_H_

#include "Vector.h"
#include "Matrix.h"

namespace arda 
   {

   ////////////////////////////////////////////////////////////////////////////////
   // General notes:
   //
   // As much as possible is defined inline for performance.
   // Using typedef'ed arrays for vectors and matrices necessitates a lot of 
   // redundant function names: scale2, scale3, scale4, for example.
   // This is, however, perfectly in line with the implementation of OpenGL and
   // allows the library to maintain optimal memory usage and performance. (I hope!)
   // In any event, I think I could do far worse than to emulate the OpenGL
   // working group - a group of people who have studied this for years and are
   // experts in the field.

   ////////////////////////////////////////////////////////////////////////////////
   // Angles

   namespace Angles
      {
      // This many decimals is enough to cover even IEEE quad precision (128 bit)
      // reals.  Run on PA-RISC lately?  :)
      const double PI = 3.14159265358979323846264338327950288;
      }

   ////////////////////////////////////////////////////////////////////////////////
   // Mixed Vector / Matrix operations.
   // * *=
   //     Defined for Vector * Matrix and Matrix * Vector, but these
   //     operators are not part of the class.  They are defined independently.

   // Vector * Matrix
   template <typename T> inline Vector2<T>& operator*=(Vector2<T>& v, const Matrix22<T>& m)
      {
      Vector2<T> vres(0);
      vres[0] = v[0]*m[0] + v[1]*m[1];
      vres[1] = v[0]*m[2] + v[1]*m[3];
      v = vres;
      return v;
      }
   template <typename T> inline Vector2<T> operator*(const Vector2<T>& v, const Matrix22<T>& m)
      { return Vector2<T>(v) *= m; }

   // Matrix * Vector
   // Note: can't define Matrix *= Vector since the result is a Vector.
   template <typename T> inline Vector2<T> operator*(const Matrix22<T>& m, const Vector2<T>& v)
      { 
      Vector2<T> vres(0);
      vres[0] = m[0]*v[0] + m[2]*v[1];
      vres[1] = m[1]*v[0] + m[3]*v[1];
      return vres;
      }


   } // namespace arda

#endif
